@@ -116,7 +116,7 @@ protected:
     */
     UInt32 		controlID;
     /*! @var channelID 
-        The ID of the channel this control affects - may be 	kIOAudioControlChannelIDAll if it represents all channels.
+        The ID of the channel this control affects - may be kIOAudioControlChannelIDAll if it represents all channels.
     */
     UInt32		channelID;
     
@@ -171,12 +171,38 @@ public:
 	// OSMetaClassDeclareReservedUsed(IOAudioControl, 2);
 	virtual void sendQueuedNotifications(void);
 
+#if !(defined(__ppc__) && defined(KPI_10_4_0_PPC_COMPAT))
+	// OSMetaClassDeclareReservedUsed(IOAudioControl, 3);
+    /*!
+     * @function createUserClient
+     * @abstract Creates a new IOAudioControlUserClient instance.
+     * @discussion This function is called by newUserClient() to create a new IOAudioControlUserClient instance.  This function may be overridden by subclasses that need to add functionality
+     *  to the IOAudioControlUserClient.  In that case, they must subclass IOAudioControlUserClient
+     *  and return a new, initialized instance of that subclass.
+	 *  A derived class that requires overriding of createUserClient should override the version with the properties
+	 *  parameter for Intel targets, and without the properties parameter for PPC targets.  The #if __i386__ directive
+	 *  can be used to select between the two behaviors.
+     * @param task The task requesting the new user client.
+     * @param securityID Optional security paramater passed in by the client - ignored.
+     * @param type Optional user client type passed in by the client.
+     * @param newUserClient The IOAudioControlUserClient * must be stored in this param on a successful
+     *  completion.
+     * @param properties A dictionary of additional properties for the connection.
+     * @result Returns kIOReturnSuccess on success.
+     */
+    virtual IOReturn createUserClient(task_t task, void *securityID, UInt32 type, IOAudioControlUserClient **newUserClient, OSDictionary *properties);
+#endif
+
 private:
     OSMetaClassDeclareReservedUsed(IOAudioControl, 0);
     OSMetaClassDeclareReservedUsed(IOAudioControl, 1);
     OSMetaClassDeclareReservedUsed(IOAudioControl, 2);
-
+#if !(defined(__ppc__) && defined(KPI_10_4_0_PPC_COMPAT))
+    OSMetaClassDeclareReservedUsed(IOAudioControl, 3);
+#else
     OSMetaClassDeclareReservedUnused(IOAudioControl, 3);
+#endif
+
     OSMetaClassDeclareReservedUnused(IOAudioControl, 4);
     OSMetaClassDeclareReservedUnused(IOAudioControl, 5);
     OSMetaClassDeclareReservedUnused(IOAudioControl, 6);
@@ -303,13 +329,18 @@ public:
      *  IOAudioControl.  This is typically done when the user process needs to register for value change
      *  notifications.  This implementation allocates a new IOAudioControlUserClient object.  There is no
      *  need to call this directly.
+	 *  A derived class that requires overriding of newUserClient should override the version with the properties
+	 *  parameter for Intel targets, and without the properties parameter for PPC targets.  The #if __i386__ directive
+	 *  can be used to select between the two behaviors.
      * @param task The task requesting the new user client.
      * @param securityID Optional security paramater passed in by the client - ignored.
      * @param type Optional user client type passed in by the client - 0 for the default user client type.
      * @param handler The new IOUserClient * must be stored in this param on a successful completion.
+     * @param properties A dictionary of additional properties for the connection.
      * @result Returns kIOReturnSuccess on success.  May also result kIOReturnError or kIOReturnNoMemory.
      */
     virtual IOReturn newUserClient(task_t task, void *securityID, UInt32 type, IOUserClient **handler);
+    virtual IOReturn newUserClient(task_t task, void *securityID, UInt32 type, OSDictionary *properties, IOUserClient **handler);
 
     /*!
      * @function createUserClient
@@ -317,6 +348,9 @@ public:
      * @discussion This function is called by newUserClient() to create a new IOAudioControlUserClient instance.  This function may be overridden by subclasses that need to add functionality
      *  to the IOAudioControlUserClient.  In that case, they must subclass IOAudioControlUserClient
      *  and return a new, initialized instance of that subclass.
+	 *  A derived class that requires overriding of createUserClient should override the version with the properties
+	 *  parameter for Intel targets, and without the properties parameter for PPC targets.  The #if __i386__ directive
+	 *  can be used to select between the two behaviors.
      * @param task The task requesting the new user client.
      * @param securityID Optional security paramater passed in by the client - ignored.
      * @param type Optional user client type passed in by the client.
