@@ -52,6 +52,7 @@ typedef struct IOAudioClientBuffer {
     struct IOAudioClientBuffer	*nextClip;
     struct IOAudioClientBuffer	*previousClip;
     struct IOAudioClientBuffer	*nextClient;
+	IOAudioBufferDataDescriptor *bufferDataDescriptor;
 } IOAudioClientBuffer;
 
 typedef struct IOAudioClientBufferExtendedInfo {
@@ -108,6 +109,10 @@ public:
 	virtual IOReturn getNearestStartTime(IOAudioStream *audioStream, IOAudioTimeStamp *ioTimeStamp, UInt32 isInput);
 	virtual IOReturn getClientNearestStartTime(IOAudioStream *audioStream, IOAudioTimeStamp *ioTimeStamp, UInt32 isInput);
 	virtual IOReturn safeRegisterClientBuffer(UInt32 audioStreamIndex, void *sourceBuffer, UInt32 bufSizeInBytes, UInt32 bufferSetID);
+#if !(defined(__ppc__) && defined(KPI_10_4_0_PPC_COMPAT))
+    // OSMetaClassDeclareReservedUsed(IOAudioEngineUserClient, 5);	
+    virtual bool initWithAudioEngine(IOAudioEngine *engine, task_t task, void *securityToken, UInt32 type, OSDictionary *properties);
+#endif
 
 private:
     OSMetaClassDeclareReservedUsed(IOAudioEngineUserClient, 0);
@@ -115,8 +120,12 @@ private:
     OSMetaClassDeclareReservedUsed(IOAudioEngineUserClient, 2);
     OSMetaClassDeclareReservedUsed(IOAudioEngineUserClient, 3);
     OSMetaClassDeclareReservedUsed(IOAudioEngineUserClient, 4);
-
+#if !(defined(__ppc__) && defined(KPI_10_4_0_PPC_COMPAT))
+    OSMetaClassDeclareReservedUsed(IOAudioEngineUserClient, 5);
+#else
     OSMetaClassDeclareReservedUnused(IOAudioEngineUserClient, 5);
+#endif
+
     OSMetaClassDeclareReservedUnused(IOAudioEngineUserClient, 6);
     OSMetaClassDeclareReservedUnused(IOAudioEngineUserClient, 7);
     OSMetaClassDeclareReservedUnused(IOAudioEngineUserClient, 8);
@@ -171,8 +180,10 @@ protected:
 public:
 
     static IOAudioEngineUserClient *withAudioEngine(IOAudioEngine *engine, task_t clientTask, void *securityToken, UInt32 type);
+    static IOAudioEngineUserClient *withAudioEngine(IOAudioEngine *engine, task_t clientTask, void *securityToken, UInt32 type, OSDictionary *properties);
 
     virtual bool initWithAudioEngine(IOAudioEngine *engine, task_t task, void *securityToken, UInt32 type);
+
     virtual void free();
     virtual void freeClientBufferSetList();
     virtual void freeClientBuffer(IOAudioClientBuffer *clientBuffer);
